<?php

/**
 * @file
 * Provides token support for protected downloads.
 */

use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Language\LanguageInterface;
use Drupal\Core\Render\BubbleableMetadata;
use Drupal\Core\StreamWrapper\StreamWrapperManager;
use Drupal\file\FileInterface;
use Drupal\protected_download\LinkGeneratorInterface;
use Drupal\protected_download\PolicyRepositoryInterface;

/**
 * Implements hook_token_info().
 */
function protected_download_token_info() {
  $file['protected-download-expire'] = [
    'name' => t('Protected download expiry date'),
    'description' => t('The date when a protected download generated now will expire.'),
  ];

  $file['protected-download-url'] = [
    'name' => t('Protected download URL'),
    'description' => t('The web-accessible URL for the file protected with an authentication code.'),
  ];

  return [
    'types' => [],
    'tokens' => [
      'file' => $file,
    ],
  ];
}

/**
 * Implements hook_tokens().
 */
function protected_download_tokens($type, $tokens, array $data, array $options, BubbleableMetadata $bubbleable_metadata) {
  $token_service = \Drupal::token();

  $url_options = ['absolute' => TRUE];
  if (isset($options['langcode'])) {
    $url_options['language'] = \Drupal::languageManager()->getLanguage($options['langcode']);
    $langcode = $options['langcode'];
  }
  else {
    $langcode = LanguageInterface::LANGCODE_DEFAULT;
  }
  $replacements = [];
  if (
    $type === 'file' &&
    isset($data['file']) &&
    $data['file'] instanceof FileInterface
  ) {
    $now = \Drupal::time()->getRequestTime();

    $file = $data['file'];
    $scheme = StreamWrapperManager::getScheme($file->getFileUri());

    $policy_repository = \Drupal::service(PolicyRepositoryInterface::class);
    assert($policy_repository instanceof PolicyRepositoryInterface);

    $policy = $policy_repository->getPolicy($scheme);
    $protected_download_expire = $policy->expire($now);

    foreach ($tokens as $name => $original) {
      switch ($name) {
        case 'protected-download-url':
          $link_generator = \Drupal::service(LinkGeneratorInterface::class);
          assert($link_generator instanceof LinkGeneratorInterface);
          $url = $link_generator->generate($file->getFileUri())
            ->setOptions($url_options)
            ->toString();
          $replacements[$original] = $url;
          break;

        case 'protected-download-expire':
          $date_formatter = \Drupal::service(DateFormatterInterface::class);
          assert($date_formatter instanceof DateFormatterInterface);
          $replacements[$original] = $date_formatter->format($protected_download_expire, 'medium', '', NULL, $langcode);
          break;
      }
    }

    if ($date_tokens = $token_service->findWithPrefix($tokens, 'protected-download-expire')) {
      $replacements += $token_service->generate('date', $date_tokens, ['date' => $protected_download_expire], $options, $bubbleable_metadata);
    }
  }

  return $replacements;
}
